/*
** Copyright (C) Red Hat, Inc. 2010
**
** This program is free software; you can redistribute it and/or modify it
** under the terms of the GNU General Public License version 2 as
** published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; see the file COPYING. If not, write to the
** Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
** MA 02139, USA.
*/

/*
 * This program is used so we can authenticate against PAM (via saslauthd)
 * from a non-root python program.
 *
 * Author: Chris Feist <cfeist@redhat.com>
 */

#include <Python.h>
#include <sasl/sasl.h>
#include <string.h>

static int sasl_getopts_callback(void *context,
                                 const char *plugin_name,
                                 const char *option,
                                 const char **result,
                                 unsigned *len)
{
    static const char authd_option[] = "pwcheck_method";
    static const char authd_result[] = "saslauthd";
    static const char authd_version_option[] = "saslauthd_version";
    static const char authd_version_result[] = "2";

    if (result) {
        *result = 0;
        if (!strcmp(option, authd_option))
            *result = authd_result;
        else if (!strcmp(option, authd_version_option))
            *result = authd_version_result;
    }

    if (len)
        *len = 0;

    return SASL_OK;
}

static sasl_callback_t callbacks[] = {
    { SASL_CB_GETOPT, (int (*)()) sasl_getopts_callback, NULL }
};

static int authenticate(const char *username,
                        const char *password,
                        const char *serverFQDN,
                        const char *user_realm,
                        const char *iplocalport,
                        const char *ipremoteport)
{
    int ret;
    sasl_conn_t *conn;

    ret = sasl_server_init(callbacks, "luci");
    if (ret != SASL_OK)
        return 0;
 
    ret = sasl_server_new("luci",
            serverFQDN, user_realm, iplocalport, ipremoteport,
            NULL, 0, &conn);

    if (ret != SASL_OK)
        return 0;

    ret = sasl_checkpass(conn, username, strlen(username),
                         password, strlen(password));
    if (ret == SASL_OK)
        return 1;

    return 0;
}

static PyObject *lucipam_checkauth(PyObject *self, PyObject *args)
{
    const char *username;
    const char *password;
    const char *serverFQDN;
    const char *user_realm;
    const char *iplocalport;
    const char *ipremoteport;

    if (!PyArg_ParseTuple(args, "sszzzz", &username, &password, &serverFQDN, &user_realm, &iplocalport, &ipremoteport))
        return NULL;

    if (authenticate(username, password, serverFQDN, user_realm, iplocalport, ipremoteport))
        return Py_BuildValue("i", 1);

    /* Failure */
    return Py_BuildValue("i", 0);
}    

static PyMethodDef LucipamMethods[] = {
    {"checkauth", lucipam_checkauth, METH_VARARGS, "Authenticate against PAM via saslauthd"},
    {NULL, NULL, 0, NULL}
};

PyMODINIT_FUNC initlucipam(void)
{
    (void) Py_InitModule("lucipam", LucipamMethods);
}
